@@ -19,10 +19,11 @@
 #ifndef CLIENT_X11_XF_VIDEO_H_
 #define CLIENT_X11_XF_VIDEO_H_
 
-#include "xfreerdp.h"
+#include <freerdp/client/video.h>
 
-#include <freerdp/channels/geometry.h>
-#include <freerdp/channels/video.h>
+#include "xf_types.h"
+
+typedef struct s_xfVideoContext xfVideoContext;
 
 void xf_video_control_init(xfContext* xfc, VideoClientContext* video);
 void xf_video_control_uninit(xfContext* xfc, VideoClientContext* video);
