@@ -226,7 +226,7 @@ static inline INT32 planar_skip_plane_rle(const BYTE* WINPR_RESTRICT pSrcData, U
 
 			if (used > SrcSize)
 			{
-				WLog_ERR(TAG, "planar plane used %" PRIu32 " exceeds SrcSize %" PRIu32, used,
+				WLog_ERR(TAG, "planar plane used %" PRIu32 " exceeds SrcSize %" PRId32, used,
 				         INT32_MAX);
 				return -1;
 			}
@@ -727,8 +727,14 @@ BOOL freerdp_bitmap_decompress_planar(BITMAP_PLANAR_CONTEXT* WINPR_RESTRICT plan
 	WINPR_ASSERT(planar);
 	WINPR_ASSERT(prims);
 
+	if (planar->maxWidth < nSrcWidth)
+		return FALSE;
+	if (planar->maxHeight < nSrcHeight)
+		return FALSE;
+
+	const UINT32 bpp = FreeRDPGetBytesPerPixel(DstFormat);
 	if (nDstStep <= 0)
-		nDstStep = nDstWidth * FreeRDPGetBytesPerPixel(DstFormat);
+		nDstStep = nDstWidth * bpp;
 
 	const BYTE* srcp = pSrcData;
 
@@ -830,8 +836,9 @@ BOOL freerdp_bitmap_decompress_planar(BITMAP_PLANAR_CONTEXT* WINPR_RESTRICT plan
 
 			if ((planes[2] + rawSizes[2]) > &pSrcData[SrcSize])
 			{
-				WLog_ERR(TAG, "plane size mismatch %p + %" PRIu32 " > %p", planes[2], rawSizes[2],
-				         &pSrcData[SrcSize]);
+				WLog_ERR(TAG, "plane size mismatch %p + %" PRIu32 " > %p",
+				         WINPR_CXX_COMPAT_CAST(const void*, planes[2]), rawSizes[2],
+				         WINPR_CXX_COMPAT_CAST(const void*, &pSrcData[SrcSize]));
 				return FALSE;
 			}
 		}
@@ -839,7 +846,7 @@ BOOL freerdp_bitmap_decompress_planar(BITMAP_PLANAR_CONTEXT* WINPR_RESTRICT plan
 		{
 			if ((SrcSize - diff) < base)
 			{
-				WLog_ERR(TAG, "plane size mismatch %" PRIu32 " < %" PRIu32, SrcSize - diff, base);
+				WLog_ERR(TAG, "plane size mismatch %" PRIuz " < %" PRIu32, SrcSize - diff, base);
 				return FALSE;
 			}
 
@@ -849,8 +856,9 @@ BOOL freerdp_bitmap_decompress_planar(BITMAP_PLANAR_CONTEXT* WINPR_RESTRICT plan
 
 			if ((planes[2] + rawSizes[2]) > &pSrcData[SrcSize])
 			{
-				WLog_ERR(TAG, "plane size mismatch %p + %" PRIu32 " > %p", planes[2], rawSizes[2],
-				         &pSrcData[SrcSize]);
+				WLog_ERR(TAG, "plane size mismatch %p + %" PRIu32 " > %p",
+				         WINPR_CXX_COMPAT_CAST(const void*, planes[2]), rawSizes[2],
+				         WINPR_CXX_COMPAT_CAST(const void*, &pSrcData[SrcSize]));
 				return FALSE;
 			}
 		}
@@ -948,6 +956,24 @@ BOOL freerdp_bitmap_decompress_planar(BITMAP_PLANAR_CONTEXT* WINPR_RESTRICT plan
 		}
 		else /* RLE */
 		{
+			if (nYDst + nSrcHeight > nTotalHeight)
+			{
+				WLog_ERR(TAG,
+				         "planar plane destination Y %" PRIu32 " + height %" PRIu32
+				         " exceeds totalHeight %" PRIu32,
+				         nYDst, nSrcHeight, nTotalHeight);
+				return FALSE;
+			}
+
+			if ((nXDst + nSrcWidth) * bpp > nDstStep)
+			{
+				WLog_ERR(TAG,
+				         "planar plane destination (X %" PRIu32 " + width %" PRIu32
+				         ") * bpp %" PRIu32 " exceeds stride %" PRIu32,
+				         nXDst, nSrcWidth, bpp, nDstStep);
+				return FALSE;
+			}
+
 			status = planar_decompress_plane_rle(
 			    planes[0], WINPR_ASSERTING_INT_CAST(uint32_t, rleSizes[0]), pTempData, nTempStep,
 			    nXDst, nYDst, nSrcWidth, nSrcHeight, 2, vFlip); /* RedPlane */
