@@ -408,7 +408,7 @@ BOOL freerdp_key_generate(rdpPrivateKey* key, const char* type, size_t count, ..
 		WLog_ERR(TAG, "Argument type=%s requires count=1, got %" PRIuz ", aborting", type, count);
 		return FALSE;
 	}
-	va_list ap;
+	va_list ap = { 0 };
 	va_start(ap, count);
 	const int key_length = va_arg(ap, int);
 	va_end(ap);
