@@ -59,7 +59,11 @@
 #endif
 
 // C23 related macros
-#if defined(__STDC_VERSION__) && (__STDC_VERSION__ >= 202311L)
+#if defined(__cplusplus) && (__cplusplus >= 201703L)
+#define WINPR_FALLTHROUGH \
+	(void)0;              \
+	[[fallthrough]];
+#elif defined(__STDC_VERSION__) && (__STDC_VERSION__ >= 201904L)
 #define WINPR_FALLTHROUGH \
 	(void)0;              \
 	[[fallthrough]];
@@ -75,6 +79,20 @@
 #define WINPR_FALLTHROUGH (void)0;
 #endif
 
+#if defined(WINPR_DEFINE_ATTR_NODISCARD)
+#if defined(__STDC_VERSION__) && (__STDC_VERSION__ >= 202003L)
+#define WINPR_ATTR_NODISCARD [[nodiscard]]
+#elif defined(__clang__)
+#define WINPR_ATTR_NODISCARD __attribute__((warn_unused_result))
+#elif defined(__GNUC__) && (__GNUC__ >= 7)
+#define WINPR_ATTR_NODISCARD __attribute__((warn_unused_result))
+#else
+#define WINPR_ATTR_NODISCARD
+#endif
+#else
+#define WINPR_ATTR_NODISCARD
+#endif
+
 #if defined(__clang__)
 #define WINPR_PRAGMA_DIAG_PUSH WINPR_DO_PRAGMA(clang diagnostic push)
 #define WINPR_PRAGMA_DIAG_IGNORED_OVERLENGTH_STRINGS \
@@ -514,21 +532,24 @@ WINPR_PRAGMA_DIAG_IGNORED_RESERVED_ID_MACRO
 
 WINPR_PRAGMA_DIAG_POP
 
-#if defined(__STDC_VERSION__) && (__STDC_VERSION__ >= 202311L)
+#if defined(__STDC_VERSION__) && (__STDC_VERSION__ >= 201904L)
 #define WINPR_DEPRECATED(obj) [[deprecated]] obj
-#define WINPR_DEPRECATED_VAR(text, obj) [[deprecated(text)]] obj
-#define WINPR_NORETURN(obj) [[noreturn]] obj
-#elif defined(WIN32) && !defined(__CYGWIN__)
-#define WINPR_DEPRECATED(obj) __declspec(deprecated) obj
-#define WINPR_DEPRECATED_VAR(text, obj) __declspec(deprecated(text)) obj
-#define WINPR_NORETURN(obj) __declspec(noreturn) obj
+#define WINPR_DEPRECATED_VAR(text, obj) [[deprecated("[deprecated] " text)]] obj
 #elif defined(__GNUC__)
 #define WINPR_DEPRECATED(obj) obj __attribute__((deprecated))
-#define WINPR_DEPRECATED_VAR(text, obj) obj __attribute__((deprecated(text)))
-#define WINPR_NORETURN(obj) __attribute__((__noreturn__)) obj
+#define WINPR_DEPRECATED_VAR(text, obj) obj __attribute__((deprecated("[deprecated] " text)))
 #else
 #define WINPR_DEPRECATED(obj) obj
 #define WINPR_DEPRECATED_VAR(text, obj) obj
+#endif
+
+#if defined(__STDC_VERSION__) && (__STDC_VERSION__ >= 202202L)
+#define WINPR_NORETURN(obj) [[noreturn]] obj
+#elif defined(WIN32) && !defined(__CYGWIN__)
+#define WINPR_NORETURN(obj) __declspec(noreturn) obj
+#elif defined(__GNUC__)
+#define WINPR_NORETURN(obj) __attribute__((__noreturn__)) obj
+#else
 #define WINPR_NORETURN(obj) obj
 #endif
 
@@ -573,7 +594,7 @@ WINPR_PRAGMA_DIAG_POP
 #if defined(__GNUC__) || defined(__clang__)
 #define WINPR_ATTR_FORMAT_ARG(pos, args) __attribute__((__format__(__printf__, pos, args)))
 #define WINPR_FORMAT_ARG /**/
-#else
+#elif defined(_MSC_VER)
 #define WINPR_ATTR_FORMAT_ARG(pos, args)
 #define WINPR_FORMAT_ARG _Printf_format_string_
 #endif
@@ -595,7 +616,12 @@ WINPR_PRAGMA_DIAG_POP
 // WARNING: *do not* use thread-local storage for new code because it is not portable
 // It is only used for VirtualChannelInit, and all FreeRDP channels use VirtualChannelInitEx
 // The old virtual channel API is only realistically used on Windows where TLS is available
-#if defined _WIN32 || defined __CYGWIN__
+#if defined(__STDC_VERSION__) && (__STDC_VERSION__ >= 201112L) && \
+    !defined(__STDC_NO_THREADS__) // C11
+#include <threads.h>
+
+#define WINPR_TLS thread_local
+#elif defined _WIN32 || defined __CYGWIN__
 #ifdef __GNUC__
 #define WINPR_TLS __thread
 #else
@@ -621,7 +647,7 @@ WINPR_PRAGMA_DIAG_POP
 
 #if defined(__cplusplus) && (__cplusplus >= 201703L)
 #define WINPR_ATTR_UNUSED [[maybe_unused]] /** @since version 3.12.0 */
-#elif defined(__STDC_VERSION__) && (__STDC_VERSION__ >= 202000L)
+#elif defined(__STDC_VERSION__) && (__STDC_VERSION__ >= 201904L)
 #define WINPR_ATTR_UNUSED [[maybe_unused]] /** @since version 3.12.0 */
 #elif defined(__GNUC__) || defined(__clang__)
 #define WINPR_ATTR_UNUSED __attribute__((unused)) /** @since version 3.12.0 */
