@@ -35,11 +35,11 @@
 #include "../NTLM/ntlm_export.h"
 #include "../Kerberos/kerberos.h"
 #include "../sspi.h"
+#include "../../utils.h"
 #include "../../log.h"
 #define TAG WINPR_TAG("negotiate")
 
-static const char NEGO_REG_KEY[] =
-    "Software\\" WINPR_VENDOR_STRING "\\" WINPR_PRODUCT_STRING "\\SSPI\\Negotiate";
+#define NEGO_REG_KEY "Software\\%s\\SSPI\\Negotiate"
 
 static const char PACKAGE_NAME_DISABLE_ALL[] = "none";
 static const char PACKAGE_NAME_NTLM[] = "ntlm";
@@ -318,7 +318,6 @@ fail:
 static BOOL negotiate_get_config(void* pAuthData, BOOL* kerberos, BOOL* ntlm, BOOL* u2u)
 {
 	HKEY hKey = NULL;
-	LONG rc = 0;
 
 	WINPR_ASSERT(kerberos);
 	WINPR_ASSERT(ntlm);
@@ -342,16 +341,22 @@ static BOOL negotiate_get_config(void* pAuthData, BOOL* kerberos, BOOL* ntlm, BO
 		return TRUE; // use explicit authentication package list
 	}
 
-	rc = RegOpenKeyExA(HKEY_LOCAL_MACHINE, NEGO_REG_KEY, 0, KEY_READ | KEY_WOW64_64KEY, &hKey);
-	if (rc == ERROR_SUCCESS)
 	{
-		DWORD dwValue = 0;
+		char* key = winpr_getApplicatonDetailsRegKey(NEGO_REG_KEY);
+		if (key)
+		{
+			const LONG rc =
+			    RegOpenKeyExA(HKEY_LOCAL_MACHINE, key, 0, KEY_READ | KEY_WOW64_64KEY, &hKey);
+			free(key);
+			if (rc == ERROR_SUCCESS)
+			{
+				DWORD dwValue = 0;
 
-		if (negotiate_get_dword(hKey, PACKAGE_NAME_KERBEROS, &dwValue))
-			*kerberos = (dwValue != 0) ? TRUE : FALSE;
+				if (negotiate_get_dword(hKey, PACKAGE_NAME_KERBEROS, &dwValue))
+					*kerberos = (dwValue != 0) ? TRUE : FALSE;
 
-		if (negotiate_get_dword(hKey, PACKAGE_NAME_KERBEROS_U2U, &dwValue))
-			*u2u = (dwValue != 0) ? TRUE : FALSE;
+				if (negotiate_get_dword(hKey, PACKAGE_NAME_KERBEROS_U2U, &dwValue))
+					*u2u = (dwValue != 0) ? TRUE : FALSE;
 
 #if !defined(WITH_KRB5_NO_NTLM_FALLBACK)
 		if (negotiate_get_dword(hKey, PACKAGE_NAME_NTLM, &dwValue))
@@ -359,6 +364,8 @@ static BOOL negotiate_get_config(void* pAuthData, BOOL* kerberos, BOOL* ntlm, BO
 #endif
 
 		RegCloseKey(hKey);
+			}
+		}
 	}
 
 	return TRUE;
@@ -404,7 +411,7 @@ static BOOL negotiate_write_neg_token(PSecBuffer output_buffer, NegToken* token)
 	{
 		if (!WinPrAsn1EncContextualRawContent(enc, 0, &mechTypes))
 			goto cleanup;
-		WLog_DBG(TAG, "\tmechTypes [0] (%li bytes)", token->mechTypes.cbBuffer);
+		WLog_DBG(TAG, "\tmechTypes [0] (%" PRIu32 " bytes)", token->mechTypes.cbBuffer);
 	}
 	/* negState [0] ENUMERATED */
 	else if (token->negState != NOSTATE)
@@ -427,7 +434,7 @@ static BOOL negotiate_write_neg_token(PSecBuffer output_buffer, NegToken* token)
 	{
 		if (WinPrAsn1EncContextualOctetString(enc, 2, &mechToken) == 0)
 			goto cleanup;
-		WLog_DBG(TAG, "\tmechToken [2] (%li bytes)", token->mechToken.cbBuffer);
+		WLog_DBG(TAG, "\tmechToken [2] (%" PRIu32 " bytes)", token->mechToken.cbBuffer);
 	}
 
 	/* mechListMIC [3] OCTET STRING */
@@ -435,7 +442,7 @@ static BOOL negotiate_write_neg_token(PSecBuffer output_buffer, NegToken* token)
 	{
 		if (WinPrAsn1EncContextualOctetString(enc, 3, &mechListMic) == 0)
 			goto cleanup;
-		WLog_DBG(TAG, "\tmechListMIC [3] (%li bytes)", token->mic.cbBuffer);
+		WLog_DBG(TAG, "\tmechListMIC [3] (%" PRIu32 " bytes)", token->mic.cbBuffer);
 	}
 
 	/* NegTokenInit or NegTokenResp */
@@ -470,13 +477,13 @@ cleanup:
 
 static BOOL negotiate_read_neg_token(PSecBuffer input, NegToken* token)
 {
-	WinPrAsn1Decoder dec;
-	WinPrAsn1Decoder dec2;
-	WinPrAsn1_OID oid;
+	WinPrAsn1Decoder dec = { .encoding = WINPR_ASN1_BER, { 0 } };
+	WinPrAsn1Decoder dec2 = { .encoding = WINPR_ASN1_BER, { 0 } };
+	WinPrAsn1_OID oid = { 0 };
 	WinPrAsn1_tagId contextual = 0;
 	WinPrAsn1_tag tag = 0;
 	size_t len = 0;
-	WinPrAsn1_OctetString octet_string;
+	WinPrAsn1_OctetString octet_string = { 0 };
 	BOOL err = 0;
 
 	WINPR_ASSERT(input);
@@ -533,7 +540,7 @@ static BOOL negotiate_read_neg_token(PSecBuffer input, NegToken* token)
 						return FALSE;
 					token->mechTypes.cbBuffer = (UINT32)mlen;
 					token->mechTypes.pvBuffer = Stream_Buffer(&s);
-					WLog_DBG(TAG, "\tmechTypes [0] (%li bytes)", token->mechTypes.cbBuffer);
+					WLog_DBG(TAG, "\tmechTypes [0] (%" PRIu32 " bytes)", token->mechTypes.cbBuffer);
 				}
 				else
 				{
@@ -564,7 +571,7 @@ static BOOL negotiate_read_neg_token(PSecBuffer input, NegToken* token)
 					/* reqFlags [1] ContextFlags BIT STRING (ignored) */
 					if (!WinPrAsn1DecPeekTagAndLen(&dec2, &tag, &len) || (tag != ER_TAG_BIT_STRING))
 						return FALSE;
-					WLog_DBG(TAG, "\treqFlags [1] (%li bytes)", len);
+					WLog_DBG(TAG, "\treqFlags [1] (%" PRIuz " bytes)", len);
 				}
 				else
 				{
@@ -584,7 +591,7 @@ static BOOL negotiate_read_neg_token(PSecBuffer input, NegToken* token)
 				token->mechToken.cbBuffer = (UINT32)octet_string.len;
 				token->mechToken.pvBuffer = octet_string.data;
 				token->mechToken.BufferType = SECBUFFER_TOKEN;
-				WLog_DBG(TAG, "\tmechToken [2] (%li bytes)", octet_string.len);
+				WLog_DBG(TAG, "\tmechToken [2] (%" PRIuz " bytes)", octet_string.len);
 				break;
 			case 3:
 				/* mechListMic [3] OCTET STRING */
@@ -595,7 +602,7 @@ static BOOL negotiate_read_neg_token(PSecBuffer input, NegToken* token)
 				token->mic.cbBuffer = (UINT32)octet_string.len;
 				token->mic.pvBuffer = octet_string.data;
 				token->mic.BufferType = SECBUFFER_TOKEN;
-				WLog_DBG(TAG, "\tmechListMIC [3] (%li bytes)", octet_string.len);
+				WLog_DBG(TAG, "\tmechListMIC [3] (%" PRIuz " bytes)", octet_string.len);
 				break;
 			default:
 				WLog_ERR(TAG, "unknown contextual item %d", contextual);
@@ -983,8 +990,8 @@ static SECURITY_STATUS SEC_ENTRY negotiate_InitializeSecurityContextA(
 
 static const Mech* guessMech(PSecBuffer input_buffer, BOOL* spNego, WinPrAsn1_OID* oid)
 {
-	WinPrAsn1Decoder decoder;
-	WinPrAsn1Decoder appDecoder;
+	WinPrAsn1Decoder decoder = { .encoding = WINPR_ASN1_BER, { 0 } };
+	WinPrAsn1Decoder appDecoder = { .encoding = WINPR_ASN1_BER, { 0 } };
 	WinPrAsn1_tagId tag = 0;
 	const char ssp[] = "NTLMSSP";
 
@@ -1032,8 +1039,8 @@ static SECURITY_STATUS SEC_ENTRY negotiate_AcceptSecurityContext(
 	SecBufferDesc mech_input = { SECBUFFER_VERSION, 1, &input_token.mechToken };
 	SecBufferDesc mech_output = { SECBUFFER_VERSION, 1, &output_token.mechToken };
 	SECURITY_STATUS status = SEC_E_INTERNAL_ERROR;
-	WinPrAsn1Decoder dec;
-	WinPrAsn1Decoder dec2;
+	WinPrAsn1Decoder dec = { .encoding = WINPR_ASN1_BER, { 0 } };
+	WinPrAsn1Decoder dec2 = { .encoding = WINPR_ASN1_BER, { 0 } };
 	WinPrAsn1_tagId tag = 0;
 	WinPrAsn1_OID oid = { 0 };
 	const Mech* first_mech = NULL;
@@ -1560,9 +1567,7 @@ static SECURITY_STATUS SEC_ENTRY negotiate_QueryCredentialsAttributesA(
 
 static SECURITY_STATUS SEC_ENTRY negotiate_FreeCredentialsHandle(PCredHandle phCredential)
 {
-	MechCred* creds = NULL;
-
-	creds = sspi_SecureHandleGetLowerPointer(phCredential);
+	MechCred* creds = sspi_SecureHandleGetLowerPointer(phCredential);
 	if (!creds)
 		return SEC_E_INVALID_HANDLE;
 
