@@ -48,12 +48,12 @@ struct s_wLogAppender
 	void* DataMessageContext;
 	void* ImageMessageContext;
 	void* PacketMessageContext;
-	WLOG_APPENDER_OPEN_FN Open;
-	WLOG_APPENDER_CLOSE_FN Close;
-	WLOG_APPENDER_WRITE_MESSAGE_FN WriteMessage;
-	WLOG_APPENDER_WRITE_DATA_MESSAGE_FN WriteDataMessage;
-	WLOG_APPENDER_WRITE_IMAGE_MESSAGE_FN WriteImageMessage;
-	WLOG_APPENDER_WRITE_PACKET_MESSAGE_FN WritePacketMessage;
+	WINPR_ATTR_NODISCARD WLOG_APPENDER_OPEN_FN Open;
+	WINPR_ATTR_NODISCARD WLOG_APPENDER_CLOSE_FN Close;
+	WINPR_ATTR_NODISCARD WLOG_APPENDER_WRITE_MESSAGE_FN WriteMessage;
+	WINPR_ATTR_NODISCARD WLOG_APPENDER_WRITE_DATA_MESSAGE_FN WriteDataMessage;
+	WINPR_ATTR_NODISCARD WLOG_APPENDER_WRITE_IMAGE_MESSAGE_FN WriteImageMessage;
+	WINPR_ATTR_NODISCARD WLOG_APPENDER_WRITE_PACKET_MESSAGE_FN WritePacketMessage;
 	WLOG_APPENDER_FREE Free;
 	WLOG_APPENDER_SET Set;
 };
@@ -75,7 +75,7 @@ struct s_wLog
 	DWORD ChildrenCount;
 	DWORD ChildrenSize;
 	CRITICAL_SECTION lock;
-	const char* (*custom)(void*);
+	WINPR_ATTR_NODISCARD const char* (*custom)(void*);
 	void* context;
 };
 
